<?php
namespace TYPO3\Flow\Utility;

/*                                                                        *
 * This script belongs to the TYPO3 Flow framework.                       *
 *                                                                        *
 * It is free software; you can redistribute it and/or modify it under    *
 * the terms of the GNU Lesser General Public License, either version 3   *
 * of the License, or (at your option) any later version.                 *
 *                                                                        *
 * The TYPO3 project - inspiring people to share!                         *
 *                                                                        */

/**
 * Utility class for converting Internet Media Types to file extensions and vice
 * versa.
 *
 * "Internet Media Types" are registered through the IANA and were formerly
 * known as "MIME types". Although "MIME" still exists in the context of emails,
 * the term is outdated when speaking about media types for the web.
 *
 * The static properties containing the actual mappings is derived from a freely
 * available list provided by the Apache HTTPD project. It is generated by a script
 * which can be found in the Scripts folder of the Flow package.
 *
 * @see http://www.iana.org/assignments/media-types/
 */
class MediaTypes {

	const PATTERN_SPLITMEDIARANGE = '/^(?P<type>(?:\*|[\.!#%&\'\`\^~\$\*\+\-\|\w]+))\/(?P<subtype>(?:\*|[\.!#%&\'\`\^~\$\*\+\-\|\w]+))(?P<parameters>.*)$/i';
	const PATTERN_SPLITMEDIATYPE = '/^(?P<type>(?:[\.!#%&\'\`\^~\$\*\+\-\|\w]+))\/(?P<subtype>(?:[\.!#%&\'\`\^~\$\*\+\-\|\w]+))(?P<parameters>.*)$/i';

	/**
	 * A map of file extensions to Internet Media Types
	 *
	 * @var array
	 */
	private static $extensionToMediaType = array(
		'3dml' => 'text/vnd.in3d.3dml',
		'3ds' => 'image/x-3ds',
		'3g2' => 'video/3gpp2',
		'3gp' => 'video/3gpp',
		'7z' => 'application/x-7z-compressed',
		'aab' => 'application/x-authorware-bin',
		'aac' => 'audio/x-aac',
		'aam' => 'application/x-authorware-map',
		'aas' => 'application/x-authorware-seg',
		'abw' => 'application/x-abiword',
		'ac' => 'application/pkix-attr-cert',
		'acc' => 'application/vnd.americandynamics.acc',
		'ace' => 'application/x-ace-compressed',
		'acu' => 'application/vnd.acucobol',
		'acutc' => 'application/vnd.acucorp',
		'adp' => 'audio/adpcm',
		'aep' => 'application/vnd.audiograph',
		'afm' => 'application/x-font-type1',
		'afp' => 'application/vnd.ibm.modcap',
		'ahead' => 'application/vnd.ahead.space',
		'ai' => 'application/postscript',
		'aif' => 'audio/x-aiff',
		'aifc' => 'audio/x-aiff',
		'aiff' => 'audio/x-aiff',
		'air' => 'application/vnd.adobe.air-application-installer-package+zip',
		'ait' => 'application/vnd.dvb.ait',
		'ami' => 'application/vnd.amiga.ami',
		'apk' => 'application/vnd.android.package-archive',
		'appcache' => 'text/cache-manifest',
		'application' => 'application/x-ms-application',
		'apr' => 'application/vnd.lotus-approach',
		'arc' => 'application/x-freearc',
		'asc' => 'application/pgp-signature',
		'asf' => 'video/x-ms-asf',
		'asm' => 'text/x-asm',
		'aso' => 'application/vnd.accpac.simply.aso',
		'asx' => 'video/x-ms-asf',
		'atc' => 'application/vnd.acucorp',
		'atom' => 'application/atom+xml',
		'atomcat' => 'application/atomcat+xml',
		'atomsvc' => 'application/atomsvc+xml',
		'atx' => 'application/vnd.antix.game-component',
		'au' => 'audio/basic',
		'avi' => 'video/x-msvideo',
		'aw' => 'application/applixware',
		'azf' => 'application/vnd.airzip.filesecure.azf',
		'azs' => 'application/vnd.airzip.filesecure.azs',
		'azw' => 'application/vnd.amazon.ebook',
		'bat' => 'application/x-msdownload',
		'bcpio' => 'application/x-bcpio',
		'bdf' => 'application/x-font-bdf',
		'bdm' => 'application/vnd.syncml.dm+wbxml',
		'bed' => 'application/vnd.realvnc.bed',
		'bh2' => 'application/vnd.fujitsu.oasysprs',
		'bin' => 'application/octet-stream',
		'blb' => 'application/x-blorb',
		'blorb' => 'application/x-blorb',
		'bmi' => 'application/vnd.bmi',
		'bmp' => 'image/bmp',
		'book' => 'application/vnd.framemaker',
		'box' => 'application/vnd.previewsystems.box',
		'boz' => 'application/x-bzip2',
		'bpk' => 'application/octet-stream',
		'btif' => 'image/prs.btif',
		'bz' => 'application/x-bzip',
		'bz2' => 'application/x-bzip2',
		'c' => 'text/x-c',
		'c11amc' => 'application/vnd.cluetrust.cartomobile-config',
		'c11amz' => 'application/vnd.cluetrust.cartomobile-config-pkg',
		'c4d' => 'application/vnd.clonk.c4group',
		'c4f' => 'application/vnd.clonk.c4group',
		'c4g' => 'application/vnd.clonk.c4group',
		'c4p' => 'application/vnd.clonk.c4group',
		'c4u' => 'application/vnd.clonk.c4group',
		'cab' => 'application/vnd.ms-cab-compressed',
		'caf' => 'audio/x-caf',
		'cap' => 'application/vnd.tcpdump.pcap',
		'car' => 'application/vnd.curl.car',
		'cat' => 'application/vnd.ms-pki.seccat',
		'cb7' => 'application/x-cbr',
		'cba' => 'application/x-cbr',
		'cbr' => 'application/x-cbr',
		'cbt' => 'application/x-cbr',
		'cbz' => 'application/x-cbr',
		'cc' => 'text/x-c',
		'cct' => 'application/x-director',
		'ccxml' => 'application/ccxml+xml',
		'cdbcmsg' => 'application/vnd.contact.cmsg',
		'cdf' => 'application/x-netcdf',
		'cdkey' => 'application/vnd.mediastation.cdkey',
		'cdmia' => 'application/cdmi-capability',
		'cdmic' => 'application/cdmi-container',
		'cdmid' => 'application/cdmi-domain',
		'cdmio' => 'application/cdmi-object',
		'cdmiq' => 'application/cdmi-queue',
		'cdx' => 'chemical/x-cdx',
		'cdxml' => 'application/vnd.chemdraw+xml',
		'cdy' => 'application/vnd.cinderella',
		'cer' => 'application/pkix-cert',
		'cfs' => 'application/x-cfs-compressed',
		'cgm' => 'image/cgm',
		'chat' => 'application/x-chat',
		'chm' => 'application/vnd.ms-htmlhelp',
		'chrt' => 'application/vnd.kde.kchart',
		'cif' => 'chemical/x-cif',
		'cii' => 'application/vnd.anser-web-certificate-issue-initiation',
		'cil' => 'application/vnd.ms-artgalry',
		'cla' => 'application/vnd.claymore',
		'class' => 'application/java-vm',
		'clkk' => 'application/vnd.crick.clicker.keyboard',
		'clkp' => 'application/vnd.crick.clicker.palette',
		'clkt' => 'application/vnd.crick.clicker.template',
		'clkw' => 'application/vnd.crick.clicker.wordbank',
		'clkx' => 'application/vnd.crick.clicker',
		'clp' => 'application/x-msclip',
		'cmc' => 'application/vnd.cosmocaller',
		'cmdf' => 'chemical/x-cmdf',
		'cml' => 'chemical/x-cml',
		'cmp' => 'application/vnd.yellowriver-custom-menu',
		'cmx' => 'image/x-cmx',
		'cod' => 'application/vnd.rim.cod',
		'com' => 'application/x-msdownload',
		'conf' => 'text/plain',
		'cpio' => 'application/x-cpio',
		'cpp' => 'text/x-c',
		'cpt' => 'application/mac-compactpro',
		'crd' => 'application/x-mscardfile',
		'crl' => 'application/pkix-crl',
		'crt' => 'application/x-x509-ca-cert',
		'cryptonote' => 'application/vnd.rig.cryptonote',
		'csh' => 'application/x-csh',
		'csml' => 'chemical/x-csml',
		'csp' => 'application/vnd.commonspace',
		'css' => 'text/css',
		'cst' => 'application/x-director',
		'csv' => 'text/csv',
		'cu' => 'application/cu-seeme',
		'curl' => 'text/vnd.curl',
		'cww' => 'application/prs.cww',
		'cxt' => 'application/x-director',
		'cxx' => 'text/x-c',
		'dae' => 'model/vnd.collada+xml',
		'daf' => 'application/vnd.mobius.daf',
		'dart' => 'application/vnd.dart',
		'dataless' => 'application/vnd.fdsn.seed',
		'davmount' => 'application/davmount+xml',
		'dbk' => 'application/docbook+xml',
		'dcr' => 'application/x-director',
		'dcurl' => 'text/vnd.curl.dcurl',
		'dd2' => 'application/vnd.oma.dd2+xml',
		'ddd' => 'application/vnd.fujixerox.ddd',
		'deb' => 'application/x-debian-package',
		'def' => 'text/plain',
		'deploy' => 'application/octet-stream',
		'der' => 'application/x-x509-ca-cert',
		'dfac' => 'application/vnd.dreamfactory',
		'dgc' => 'application/x-dgc-compressed',
		'dic' => 'text/x-c',
		'dir' => 'application/x-director',
		'dis' => 'application/vnd.mobius.dis',
		'dist' => 'application/octet-stream',
		'distz' => 'application/octet-stream',
		'djv' => 'image/vnd.djvu',
		'djvu' => 'image/vnd.djvu',
		'dll' => 'application/x-msdownload',
		'dmg' => 'application/x-apple-diskimage',
		'dmp' => 'application/vnd.tcpdump.pcap',
		'dms' => 'application/octet-stream',
		'dna' => 'application/vnd.dna',
		'doc' => 'application/msword',
		'docm' => 'application/vnd.ms-word.document.macroenabled.12',
		'docx' => 'application/vnd.openxmlformats-officedocument.wordprocessingml.document',
		'dot' => 'application/msword',
		'dotm' => 'application/vnd.ms-word.template.macroenabled.12',
		'dotx' => 'application/vnd.openxmlformats-officedocument.wordprocessingml.template',
		'dp' => 'application/vnd.osgi.dp',
		'dpg' => 'application/vnd.dpgraph',
		'dra' => 'audio/vnd.dra',
		'dsc' => 'text/prs.lines.tag',
		'dssc' => 'application/dssc+der',
		'dtb' => 'application/x-dtbook+xml',
		'dtd' => 'application/xml-dtd',
		'dts' => 'audio/vnd.dts',
		'dtshd' => 'audio/vnd.dts.hd',
		'dump' => 'application/octet-stream',
		'dvb' => 'video/vnd.dvb.file',
		'dvi' => 'application/x-dvi',
		'dwf' => 'model/vnd.dwf',
		'dwg' => 'image/vnd.dwg',
		'dxf' => 'image/vnd.dxf',
		'dxp' => 'application/vnd.spotfire.dxp',
		'dxr' => 'application/x-director',
		'ecelp4800' => 'audio/vnd.nuera.ecelp4800',
		'ecelp7470' => 'audio/vnd.nuera.ecelp7470',
		'ecelp9600' => 'audio/vnd.nuera.ecelp9600',
		'ecma' => 'application/ecmascript',
		'edm' => 'application/vnd.novadigm.edm',
		'edx' => 'application/vnd.novadigm.edx',
		'efif' => 'application/vnd.picsel',
		'ei6' => 'application/vnd.pg.osasli',
		'elc' => 'application/octet-stream',
		'emf' => 'application/x-msmetafile',
		'eml' => 'message/rfc822',
		'emma' => 'application/emma+xml',
		'emz' => 'application/x-msmetafile',
		'eol' => 'audio/vnd.digital-winds',
		'eot' => 'application/vnd.ms-fontobject',
		'eps' => 'application/postscript',
		'epub' => 'application/epub+zip',
		'es3' => 'application/vnd.eszigno3+xml',
		'esa' => 'application/vnd.osgi.subsystem',
		'esf' => 'application/vnd.epson.esf',
		'et3' => 'application/vnd.eszigno3+xml',
		'etx' => 'text/x-setext',
		'eva' => 'application/x-eva',
		'evy' => 'application/x-envoy',
		'exe' => 'application/x-msdownload',
		'exi' => 'application/exi',
		'ext' => 'application/vnd.novadigm.ext',
		'ez' => 'application/andrew-inset',
		'ez2' => 'application/vnd.ezpix-album',
		'ez3' => 'application/vnd.ezpix-package',
		'f' => 'text/x-fortran',
		'f4v' => 'video/x-f4v',
		'f77' => 'text/x-fortran',
		'f90' => 'text/x-fortran',
		'fbs' => 'image/vnd.fastbidsheet',
		'fcdt' => 'application/vnd.adobe.formscentral.fcdt',
		'fcs' => 'application/vnd.isac.fcs',
		'fdf' => 'application/vnd.fdf',
		'fe_launch' => 'application/vnd.denovo.fcselayout-link',
		'fg5' => 'application/vnd.fujitsu.oasysgp',
		'fgd' => 'application/x-director',
		'fh' => 'image/x-freehand',
		'fh4' => 'image/x-freehand',
		'fh5' => 'image/x-freehand',
		'fh7' => 'image/x-freehand',
		'fhc' => 'image/x-freehand',
		'fig' => 'application/x-xfig',
		'flac' => 'audio/x-flac',
		'fli' => 'video/x-fli',
		'flo' => 'application/vnd.micrografx.flo',
		'flv' => 'video/x-flv',
		'flw' => 'application/vnd.kde.kivio',
		'flx' => 'text/vnd.fmi.flexstor',
		'fly' => 'text/vnd.fly',
		'fm' => 'application/vnd.framemaker',
		'fnc' => 'application/vnd.frogans.fnc',
		'for' => 'text/x-fortran',
		'fpx' => 'image/vnd.fpx',
		'frame' => 'application/vnd.framemaker',
		'fsc' => 'application/vnd.fsc.weblaunch',
		'fst' => 'image/vnd.fst',
		'ftc' => 'application/vnd.fluxtime.clip',
		'fti' => 'application/vnd.anser-web-funds-transfer-initiation',
		'fvt' => 'video/vnd.fvt',
		'fxp' => 'application/vnd.adobe.fxp',
		'fxpl' => 'application/vnd.adobe.fxp',
		'fzs' => 'application/vnd.fuzzysheet',
		'g2w' => 'application/vnd.geoplan',
		'g3' => 'image/g3fax',
		'g3w' => 'application/vnd.geospace',
		'gac' => 'application/vnd.groove-account',
		'gam' => 'application/x-tads',
		'gbr' => 'application/rpki-ghostbusters',
		'gca' => 'application/x-gca-compressed',
		'gdl' => 'model/vnd.gdl',
		'geo' => 'application/vnd.dynageo',
		'gex' => 'application/vnd.geometry-explorer',
		'ggb' => 'application/vnd.geogebra.file',
		'ggt' => 'application/vnd.geogebra.tool',
		'ghf' => 'application/vnd.groove-help',
		'gif' => 'image/gif',
		'gim' => 'application/vnd.groove-identity-message',
		'gml' => 'application/gml+xml',
		'gmx' => 'application/vnd.gmx',
		'gnumeric' => 'application/x-gnumeric',
		'gph' => 'application/vnd.flographit',
		'gpx' => 'application/gpx+xml',
		'gqf' => 'application/vnd.grafeq',
		'gqs' => 'application/vnd.grafeq',
		'gram' => 'application/srgs',
		'gramps' => 'application/x-gramps-xml',
		'gre' => 'application/vnd.geometry-explorer',
		'grv' => 'application/vnd.groove-injector',
		'grxml' => 'application/srgs+xml',
		'gsf' => 'application/x-font-ghostscript',
		'gtar' => 'application/x-gtar',
		'gtm' => 'application/vnd.groove-tool-message',
		'gtw' => 'model/vnd.gtw',
		'gv' => 'text/vnd.graphviz',
		'gxf' => 'application/gxf',
		'gxt' => 'application/vnd.geonext',
		'h' => 'text/x-c',
		'h261' => 'video/h261',
		'h263' => 'video/h263',
		'h264' => 'video/h264',
		'hal' => 'application/vnd.hal+xml',
		'hbci' => 'application/vnd.hbci',
		'hdf' => 'application/x-hdf',
		'hh' => 'text/x-c',
		'hlp' => 'application/winhlp',
		'hpgl' => 'application/vnd.hp-hpgl',
		'hpid' => 'application/vnd.hp-hpid',
		'hps' => 'application/vnd.hp-hps',
		'hqx' => 'application/mac-binhex40',
		'htke' => 'application/vnd.kenameaapp',
		'htm' => 'text/html',
		'html' => 'text/html',
		'hvd' => 'application/vnd.yamaha.hv-dic',
		'hvp' => 'application/vnd.yamaha.hv-voice',
		'hvs' => 'application/vnd.yamaha.hv-script',
		'i2g' => 'application/vnd.intergeo',
		'icc' => 'application/vnd.iccprofile',
		'ice' => 'x-conference/x-cooltalk',
		'icm' => 'application/vnd.iccprofile',
		'ico' => 'image/x-icon',
		'ics' => 'text/calendar',
		'ief' => 'image/ief',
		'ifb' => 'text/calendar',
		'ifm' => 'application/vnd.shana.informed.formdata',
		'iges' => 'model/iges',
		'igl' => 'application/vnd.igloader',
		'igm' => 'application/vnd.insors.igm',
		'igs' => 'model/iges',
		'igx' => 'application/vnd.micrografx.igx',
		'iif' => 'application/vnd.shana.informed.interchange',
		'imp' => 'application/vnd.accpac.simply.imp',
		'ims' => 'application/vnd.ms-ims',
		'in' => 'text/plain',
		'ink' => 'application/inkml+xml',
		'inkml' => 'application/inkml+xml',
		'install' => 'application/x-install-instructions',
		'iota' => 'application/vnd.astraea-software.iota',
		'ipfix' => 'application/ipfix',
		'ipk' => 'application/vnd.shana.informed.package',
		'irm' => 'application/vnd.ibm.rights-management',
		'irp' => 'application/vnd.irepository.package+xml',
		'iso' => 'application/x-iso9660-image',
		'itp' => 'application/vnd.shana.informed.formtemplate',
		'ivp' => 'application/vnd.immervision-ivp',
		'ivu' => 'application/vnd.immervision-ivu',
		'jad' => 'text/vnd.sun.j2me.app-descriptor',
		'jam' => 'application/vnd.jam',
		'jar' => 'application/java-archive',
		'java' => 'text/x-java-source',
		'jisp' => 'application/vnd.jisp',
		'jlt' => 'application/vnd.hp-jlyt',
		'jnlp' => 'application/x-java-jnlp-file',
		'joda' => 'application/vnd.joost.joda-archive',
		'jpe' => 'image/jpeg',
		'jpeg' => 'image/jpeg',
		'jpg' => 'image/jpeg',
		'jpgm' => 'video/jpm',
		'jpgv' => 'video/jpeg',
		'jpm' => 'video/jpm',
		'js' => 'application/javascript',
		'json' => 'application/json',
		'jsonml' => 'application/jsonml+json',
		'kar' => 'audio/midi',
		'karbon' => 'application/vnd.kde.karbon',
		'kfo' => 'application/vnd.kde.kformula',
		'kia' => 'application/vnd.kidspiration',
		'kml' => 'application/vnd.google-earth.kml+xml',
		'kmz' => 'application/vnd.google-earth.kmz',
		'kne' => 'application/vnd.kinar',
		'knp' => 'application/vnd.kinar',
		'kon' => 'application/vnd.kde.kontour',
		'kpr' => 'application/vnd.kde.kpresenter',
		'kpt' => 'application/vnd.kde.kpresenter',
		'kpxx' => 'application/vnd.ds-keypoint',
		'ksp' => 'application/vnd.kde.kspread',
		'ktr' => 'application/vnd.kahootz',
		'ktx' => 'image/ktx',
		'ktz' => 'application/vnd.kahootz',
		'kwd' => 'application/vnd.kde.kword',
		'kwt' => 'application/vnd.kde.kword',
		'lasxml' => 'application/vnd.las.las+xml',
		'latex' => 'application/x-latex',
		'lbd' => 'application/vnd.llamagraphics.life-balance.desktop',
		'lbe' => 'application/vnd.llamagraphics.life-balance.exchange+xml',
		'les' => 'application/vnd.hhe.lesson-player',
		'lha' => 'application/x-lzh-compressed',
		'link66' => 'application/vnd.route66.link66+xml',
		'list' => 'text/plain',
		'list3820' => 'application/vnd.ibm.modcap',
		'listafp' => 'application/vnd.ibm.modcap',
		'lnk' => 'application/x-ms-shortcut',
		'log' => 'text/plain',
		'lostxml' => 'application/lost+xml',
		'lrf' => 'application/octet-stream',
		'lrm' => 'application/vnd.ms-lrm',
		'ltf' => 'application/vnd.frogans.ltf',
		'lvp' => 'audio/vnd.lucent.voice',
		'lwp' => 'application/vnd.lotus-wordpro',
		'lzh' => 'application/x-lzh-compressed',
		'm13' => 'application/x-msmediaview',
		'm14' => 'application/x-msmediaview',
		'm1v' => 'video/mpeg',
		'm21' => 'application/mp21',
		'm2a' => 'audio/mpeg',
		'm2v' => 'video/mpeg',
		'm3a' => 'audio/mpeg',
		'm3u' => 'audio/x-mpegurl',
		'm3u8' => 'application/vnd.apple.mpegurl',
		'm4u' => 'video/vnd.mpegurl',
		'm4v' => 'video/x-m4v',
		'ma' => 'application/mathematica',
		'mads' => 'application/mads+xml',
		'mag' => 'application/vnd.ecowin.chart',
		'maker' => 'application/vnd.framemaker',
		'man' => 'text/troff',
		'mar' => 'application/octet-stream',
		'mathml' => 'application/mathml+xml',
		'mb' => 'application/mathematica',
		'mbk' => 'application/vnd.mobius.mbk',
		'mbox' => 'application/mbox',
		'mc1' => 'application/vnd.medcalcdata',
		'mcd' => 'application/vnd.mcd',
		'mcurl' => 'text/vnd.curl.mcurl',
		'mdb' => 'application/x-msaccess',
		'mdi' => 'image/vnd.ms-modi',
		'me' => 'text/troff',
		'mesh' => 'model/mesh',
		'meta4' => 'application/metalink4+xml',
		'metalink' => 'application/metalink+xml',
		'mets' => 'application/mets+xml',
		'mfm' => 'application/vnd.mfmp',
		'mft' => 'application/rpki-manifest',
		'mgp' => 'application/vnd.osgeo.mapguide.package',
		'mgz' => 'application/vnd.proteus.magazine',
		'mid' => 'audio/midi',
		'midi' => 'audio/midi',
		'mie' => 'application/x-mie',
		'mif' => 'application/vnd.mif',
		'mime' => 'message/rfc822',
		'mj2' => 'video/mj2',
		'mjp2' => 'video/mj2',
		'mk3d' => 'video/x-matroska',
		'mka' => 'audio/x-matroska',
		'mks' => 'video/x-matroska',
		'mkv' => 'video/x-matroska',
		'mlp' => 'application/vnd.dolby.mlp',
		'mmd' => 'application/vnd.chipnuts.karaoke-mmd',
		'mmf' => 'application/vnd.smaf',
		'mmr' => 'image/vnd.fujixerox.edmics-mmr',
		'mng' => 'video/x-mng',
		'mny' => 'application/x-msmoney',
		'mobi' => 'application/x-mobipocket-ebook',
		'mods' => 'application/mods+xml',
		'mov' => 'video/quicktime',
		'movie' => 'video/x-sgi-movie',
		'mp2' => 'audio/mpeg',
		'mp21' => 'application/mp21',
		'mp2a' => 'audio/mpeg',
		'mp3' => 'audio/mpeg',
		'mp4' => 'video/mp4',
		'mp4a' => 'audio/mp4',
		'mp4s' => 'application/mp4',
		'mp4v' => 'video/mp4',
		'mpc' => 'application/vnd.mophun.certificate',
		'mpe' => 'video/mpeg',
		'mpeg' => 'video/mpeg',
		'mpg' => 'video/mpeg',
		'mpg4' => 'video/mp4',
		'mpga' => 'audio/mpeg',
		'mpkg' => 'application/vnd.apple.installer+xml',
		'mpm' => 'application/vnd.blueice.multipass',
		'mpn' => 'application/vnd.mophun.application',
		'mpp' => 'application/vnd.ms-project',
		'mpt' => 'application/vnd.ms-project',
		'mpy' => 'application/vnd.ibm.minipay',
		'mqy' => 'application/vnd.mobius.mqy',
		'mrc' => 'application/marc',
		'mrcx' => 'application/marcxml+xml',
		'ms' => 'text/troff',
		'mscml' => 'application/mediaservercontrol+xml',
		'mseed' => 'application/vnd.fdsn.mseed',
		'mseq' => 'application/vnd.mseq',
		'msf' => 'application/vnd.epson.msf',
		'msh' => 'model/mesh',
		'msi' => 'application/x-msdownload',
		'msl' => 'application/vnd.mobius.msl',
		'msty' => 'application/vnd.muvee.style',
		'mts' => 'model/vnd.mts',
		'mus' => 'application/vnd.musician',
		'musicxml' => 'application/vnd.recordare.musicxml+xml',
		'mvb' => 'application/x-msmediaview',
		'mwf' => 'application/vnd.mfer',
		'mxf' => 'application/mxf',
		'mxl' => 'application/vnd.recordare.musicxml',
		'mxml' => 'application/xv+xml',
		'mxs' => 'application/vnd.triscape.mxs',
		'mxu' => 'video/vnd.mpegurl',
		'n-gage' => 'application/vnd.nokia.n-gage.symbian.install',
		'n3' => 'text/n3',
		'nb' => 'application/mathematica',
		'nbp' => 'application/vnd.wolfram.player',
		'nc' => 'application/x-netcdf',
		'ncx' => 'application/x-dtbncx+xml',
		'nfo' => 'text/x-nfo',
		'ngdat' => 'application/vnd.nokia.n-gage.data',
		'nitf' => 'application/vnd.nitf',
		'nlu' => 'application/vnd.neurolanguage.nlu',
		'nml' => 'application/vnd.enliven',
		'nnd' => 'application/vnd.noblenet-directory',
		'nns' => 'application/vnd.noblenet-sealer',
		'nnw' => 'application/vnd.noblenet-web',
		'npx' => 'image/vnd.net-fpx',
		'nsc' => 'application/x-conference',
		'nsf' => 'application/vnd.lotus-notes',
		'ntf' => 'application/vnd.nitf',
		'nzb' => 'application/x-nzb',
		'oa2' => 'application/vnd.fujitsu.oasys2',
		'oa3' => 'application/vnd.fujitsu.oasys3',
		'oas' => 'application/vnd.fujitsu.oasys',
		'obd' => 'application/x-msbinder',
		'obj' => 'application/x-tgif',
		'oda' => 'application/oda',
		'odb' => 'application/vnd.oasis.opendocument.database',
		'odc' => 'application/vnd.oasis.opendocument.chart',
		'odf' => 'application/vnd.oasis.opendocument.formula',
		'odft' => 'application/vnd.oasis.opendocument.formula-template',
		'odg' => 'application/vnd.oasis.opendocument.graphics',
		'odi' => 'application/vnd.oasis.opendocument.image',
		'odm' => 'application/vnd.oasis.opendocument.text-master',
		'odp' => 'application/vnd.oasis.opendocument.presentation',
		'ods' => 'application/vnd.oasis.opendocument.spreadsheet',
		'odt' => 'application/vnd.oasis.opendocument.text',
		'oga' => 'audio/ogg',
		'ogg' => 'audio/ogg',
		'ogv' => 'video/ogg',
		'ogx' => 'application/ogg',
		'omdoc' => 'application/omdoc+xml',
		'onepkg' => 'application/onenote',
		'onetmp' => 'application/onenote',
		'onetoc' => 'application/onenote',
		'onetoc2' => 'application/onenote',
		'opf' => 'application/oebps-package+xml',
		'opml' => 'text/x-opml',
		'oprc' => 'application/vnd.palm',
		'org' => 'application/vnd.lotus-organizer',
		'osf' => 'application/vnd.yamaha.openscoreformat',
		'osfpvg' => 'application/vnd.yamaha.openscoreformat.osfpvg+xml',
		'otc' => 'application/vnd.oasis.opendocument.chart-template',
		'otf' => 'application/x-font-otf',
		'otg' => 'application/vnd.oasis.opendocument.graphics-template',
		'oth' => 'application/vnd.oasis.opendocument.text-web',
		'oti' => 'application/vnd.oasis.opendocument.image-template',
		'otp' => 'application/vnd.oasis.opendocument.presentation-template',
		'ots' => 'application/vnd.oasis.opendocument.spreadsheet-template',
		'ott' => 'application/vnd.oasis.opendocument.text-template',
		'oxps' => 'application/oxps',
		'oxt' => 'application/vnd.openofficeorg.extension',
		'p' => 'text/x-pascal',
		'p10' => 'application/pkcs10',
		'p12' => 'application/x-pkcs12',
		'p7b' => 'application/x-pkcs7-certificates',
		'p7c' => 'application/pkcs7-mime',
		'p7m' => 'application/pkcs7-mime',
		'p7r' => 'application/x-pkcs7-certreqresp',
		'p7s' => 'application/pkcs7-signature',
		'p8' => 'application/pkcs8',
		'pas' => 'text/x-pascal',
		'paw' => 'application/vnd.pawaafile',
		'pbd' => 'application/vnd.powerbuilder6',
		'pbm' => 'image/x-portable-bitmap',
		'pcap' => 'application/vnd.tcpdump.pcap',
		'pcf' => 'application/x-font-pcf',
		'pcl' => 'application/vnd.hp-pcl',
		'pclxl' => 'application/vnd.hp-pclxl',
		'pct' => 'image/x-pict',
		'pcurl' => 'application/vnd.curl.pcurl',
		'pcx' => 'image/x-pcx',
		'pdb' => 'application/vnd.palm',
		'pdf' => 'application/pdf',
		'pfa' => 'application/x-font-type1',
		'pfb' => 'application/x-font-type1',
		'pfm' => 'application/x-font-type1',
		'pfr' => 'application/font-tdpfr',
		'pfx' => 'application/x-pkcs12',
		'pgm' => 'image/x-portable-graymap',
		'pgn' => 'application/x-chess-pgn',
		'pgp' => 'application/pgp-encrypted',
		'pic' => 'image/x-pict',
		'pkg' => 'application/octet-stream',
		'pki' => 'application/pkixcmp',
		'pkipath' => 'application/pkix-pkipath',
		'plb' => 'application/vnd.3gpp.pic-bw-large',
		'plc' => 'application/vnd.mobius.plc',
		'plf' => 'application/vnd.pocketlearn',
		'pls' => 'application/pls+xml',
		'pml' => 'application/vnd.ctc-posml',
		'png' => 'image/png',
		'pnm' => 'image/x-portable-anymap',
		'portpkg' => 'application/vnd.macports.portpkg',
		'pot' => 'application/vnd.ms-powerpoint',
		'potm' => 'application/vnd.ms-powerpoint.template.macroenabled.12',
		'potx' => 'application/vnd.openxmlformats-officedocument.presentationml.template',
		'ppam' => 'application/vnd.ms-powerpoint.addin.macroenabled.12',
		'ppd' => 'application/vnd.cups-ppd',
		'ppm' => 'image/x-portable-pixmap',
		'pps' => 'application/vnd.ms-powerpoint',
		'ppsm' => 'application/vnd.ms-powerpoint.slideshow.macroenabled.12',
		'ppsx' => 'application/vnd.openxmlformats-officedocument.presentationml.slideshow',
		'ppt' => 'application/vnd.ms-powerpoint',
		'pptm' => 'application/vnd.ms-powerpoint.presentation.macroenabled.12',
		'pptx' => 'application/vnd.openxmlformats-officedocument.presentationml.presentation',
		'pqa' => 'application/vnd.palm',
		'prc' => 'application/x-mobipocket-ebook',
		'pre' => 'application/vnd.lotus-freelance',
		'prf' => 'application/pics-rules',
		'ps' => 'application/postscript',
		'psb' => 'application/vnd.3gpp.pic-bw-small',
		'psd' => 'image/vnd.adobe.photoshop',
		'psf' => 'application/x-font-linux-psf',
		'pskcxml' => 'application/pskc+xml',
		'ptid' => 'application/vnd.pvi.ptid1',
		'pub' => 'application/x-mspublisher',
		'pvb' => 'application/vnd.3gpp.pic-bw-var',
		'pwn' => 'application/vnd.3m.post-it-notes',
		'pya' => 'audio/vnd.ms-playready.media.pya',
		'pyv' => 'video/vnd.ms-playready.media.pyv',
		'qam' => 'application/vnd.epson.quickanime',
		'qbo' => 'application/vnd.intu.qbo',
		'qfx' => 'application/vnd.intu.qfx',
		'qps' => 'application/vnd.publishare-delta-tree',
		'qt' => 'video/quicktime',
		'qwd' => 'application/vnd.quark.quarkxpress',
		'qwt' => 'application/vnd.quark.quarkxpress',
		'qxb' => 'application/vnd.quark.quarkxpress',
		'qxd' => 'application/vnd.quark.quarkxpress',
		'qxl' => 'application/vnd.quark.quarkxpress',
		'qxt' => 'application/vnd.quark.quarkxpress',
		'ra' => 'audio/x-pn-realaudio',
		'ram' => 'audio/x-pn-realaudio',
		'rar' => 'application/x-rar-compressed',
		'ras' => 'image/x-cmu-raster',
		'rcprofile' => 'application/vnd.ipunplugged.rcprofile',
		'rdf' => 'application/rdf+xml',
		'rdz' => 'application/vnd.data-vision.rdz',
		'rep' => 'application/vnd.businessobjects',
		'res' => 'application/x-dtbresource+xml',
		'rgb' => 'image/x-rgb',
		'rif' => 'application/reginfo+xml',
		'rip' => 'audio/vnd.rip',
		'ris' => 'application/x-research-info-systems',
		'rl' => 'application/resource-lists+xml',
		'rlc' => 'image/vnd.fujixerox.edmics-rlc',
		'rld' => 'application/resource-lists-diff+xml',
		'rm' => 'application/vnd.rn-realmedia',
		'rmi' => 'audio/midi',
		'rmp' => 'audio/x-pn-realaudio-plugin',
		'rms' => 'application/vnd.jcp.javame.midlet-rms',
		'rmvb' => 'application/vnd.rn-realmedia-vbr',
		'rnc' => 'application/relax-ng-compact-syntax',
		'roa' => 'application/rpki-roa',
		'roff' => 'text/troff',
		'rp9' => 'application/vnd.cloanto.rp9',
		'rpss' => 'application/vnd.nokia.radio-presets',
		'rpst' => 'application/vnd.nokia.radio-preset',
		'rq' => 'application/sparql-query',
		'rs' => 'application/rls-services+xml',
		'rsd' => 'application/rsd+xml',
		'rss' => 'application/rss+xml',
		'rtf' => 'application/rtf',
		'rtx' => 'text/richtext',
		's' => 'text/x-asm',
		's3m' => 'audio/s3m',
		'saf' => 'application/vnd.yamaha.smaf-audio',
		'sbml' => 'application/sbml+xml',
		'sc' => 'application/vnd.ibm.secure-container',
		'scd' => 'application/x-msschedule',
		'scm' => 'application/vnd.lotus-screencam',
		'scq' => 'application/scvp-cv-request',
		'scs' => 'application/scvp-cv-response',
		'scurl' => 'text/vnd.curl.scurl',
		'sda' => 'application/vnd.stardivision.draw',
		'sdc' => 'application/vnd.stardivision.calc',
		'sdd' => 'application/vnd.stardivision.impress',
		'sdkd' => 'application/vnd.solent.sdkm+xml',
		'sdkm' => 'application/vnd.solent.sdkm+xml',
		'sdp' => 'application/sdp',
		'sdw' => 'application/vnd.stardivision.writer',
		'see' => 'application/vnd.seemail',
		'seed' => 'application/vnd.fdsn.seed',
		'sema' => 'application/vnd.sema',
		'semd' => 'application/vnd.semd',
		'semf' => 'application/vnd.semf',
		'ser' => 'application/java-serialized-object',
		'setpay' => 'application/set-payment-initiation',
		'setreg' => 'application/set-registration-initiation',
		'sfd-hdstx' => 'application/vnd.hydrostatix.sof-data',
		'sfs' => 'application/vnd.spotfire.sfs',
		'sfv' => 'text/x-sfv',
		'sgi' => 'image/sgi',
		'sgl' => 'application/vnd.stardivision.writer-global',
		'sgm' => 'text/sgml',
		'sgml' => 'text/sgml',
		'sh' => 'application/x-sh',
		'shar' => 'application/x-shar',
		'shf' => 'application/shf+xml',
		'sid' => 'image/x-mrsid-image',
		'sig' => 'application/pgp-signature',
		'sil' => 'audio/silk',
		'silo' => 'model/mesh',
		'sis' => 'application/vnd.symbian.install',
		'sisx' => 'application/vnd.symbian.install',
		'sit' => 'application/x-stuffit',
		'sitx' => 'application/x-stuffitx',
		'skd' => 'application/vnd.koan',
		'skm' => 'application/vnd.koan',
		'skp' => 'application/vnd.koan',
		'skt' => 'application/vnd.koan',
		'sldm' => 'application/vnd.ms-powerpoint.slide.macroenabled.12',
		'sldx' => 'application/vnd.openxmlformats-officedocument.presentationml.slide',
		'slt' => 'application/vnd.epson.salt',
		'sm' => 'application/vnd.stepmania.stepchart',
		'smf' => 'application/vnd.stardivision.math',
		'smi' => 'application/smil+xml',
		'smil' => 'application/smil+xml',
		'smv' => 'video/x-smv',
		'smzip' => 'application/vnd.stepmania.package',
		'snd' => 'audio/basic',
		'snf' => 'application/x-font-snf',
		'so' => 'application/octet-stream',
		'spc' => 'application/x-pkcs7-certificates',
		'spf' => 'application/vnd.yamaha.smaf-phrase',
		'spl' => 'application/x-futuresplash',
		'spot' => 'text/vnd.in3d.spot',
		'spp' => 'application/scvp-vp-response',
		'spq' => 'application/scvp-vp-request',
		'spx' => 'audio/ogg',
		'sql' => 'application/x-sql',
		'src' => 'application/x-wais-source',
		'srt' => 'application/x-subrip',
		'sru' => 'application/sru+xml',
		'srx' => 'application/sparql-results+xml',
		'ssdl' => 'application/ssdl+xml',
		'sse' => 'application/vnd.kodak-descriptor',
		'ssf' => 'application/vnd.epson.ssf',
		'ssml' => 'application/ssml+xml',
		'st' => 'application/vnd.sailingtracker.track',
		'stc' => 'application/vnd.sun.xml.calc.template',
		'std' => 'application/vnd.sun.xml.draw.template',
		'stf' => 'application/vnd.wt.stf',
		'sti' => 'application/vnd.sun.xml.impress.template',
		'stk' => 'application/hyperstudio',
		'stl' => 'application/vnd.ms-pki.stl',
		'str' => 'application/vnd.pg.format',
		'stw' => 'application/vnd.sun.xml.writer.template',
		'sub' => 'text/vnd.dvb.subtitle',
		'sus' => 'application/vnd.sus-calendar',
		'susp' => 'application/vnd.sus-calendar',
		'sv4cpio' => 'application/x-sv4cpio',
		'sv4crc' => 'application/x-sv4crc',
		'svc' => 'application/vnd.dvb.service',
		'svd' => 'application/vnd.svd',
		'svg' => 'image/svg+xml',
		'svgz' => 'image/svg+xml',
		'swa' => 'application/x-director',
		'swf' => 'application/x-shockwave-flash',
		'swi' => 'application/vnd.aristanetworks.swi',
		'sxc' => 'application/vnd.sun.xml.calc',
		'sxd' => 'application/vnd.sun.xml.draw',
		'sxg' => 'application/vnd.sun.xml.writer.global',
		'sxi' => 'application/vnd.sun.xml.impress',
		'sxm' => 'application/vnd.sun.xml.math',
		'sxw' => 'application/vnd.sun.xml.writer',
		't' => 'text/troff',
		't3' => 'application/x-t3vm-image',
		'taglet' => 'application/vnd.mynfc',
		'tao' => 'application/vnd.tao.intent-module-archive',
		'tar' => 'application/x-tar',
		'tcap' => 'application/vnd.3gpp2.tcap',
		'tcl' => 'application/x-tcl',
		'teacher' => 'application/vnd.smart.teacher',
		'tei' => 'application/tei+xml',
		'teicorpus' => 'application/tei+xml',
		'tex' => 'application/x-tex',
		'texi' => 'application/x-texinfo',
		'texinfo' => 'application/x-texinfo',
		'text' => 'text/plain',
		'tfi' => 'application/thraud+xml',
		'tfm' => 'application/x-tex-tfm',
		'tga' => 'image/x-tga',
		'thmx' => 'application/vnd.ms-officetheme',
		'tif' => 'image/tiff',
		'tiff' => 'image/tiff',
		'tmo' => 'application/vnd.tmobile-livetv',
		'torrent' => 'application/x-bittorrent',
		'tpl' => 'application/vnd.groove-tool-template',
		'tpt' => 'application/vnd.trid.tpt',
		'tr' => 'text/troff',
		'tra' => 'application/vnd.trueapp',
		'trm' => 'application/x-msterminal',
		'tsd' => 'application/timestamped-data',
		'tsv' => 'text/tab-separated-values',
		'ttc' => 'application/x-font-ttf',
		'ttf' => 'application/x-font-ttf',
		'ttl' => 'text/turtle',
		'twd' => 'application/vnd.simtech-mindmapper',
		'twds' => 'application/vnd.simtech-mindmapper',
		'txd' => 'application/vnd.genomatix.tuxedo',
		'txf' => 'application/vnd.mobius.txf',
		'txt' => 'text/plain',
		'u32' => 'application/x-authorware-bin',
		'udeb' => 'application/x-debian-package',
		'ufd' => 'application/vnd.ufdl',
		'ufdl' => 'application/vnd.ufdl',
		'ulx' => 'application/x-glulx',
		'umj' => 'application/vnd.umajin',
		'unityweb' => 'application/vnd.unity',
		'uoml' => 'application/vnd.uoml+xml',
		'uri' => 'text/uri-list',
		'uris' => 'text/uri-list',
		'urls' => 'text/uri-list',
		'ustar' => 'application/x-ustar',
		'utz' => 'application/vnd.uiq.theme',
		'uu' => 'text/x-uuencode',
		'uva' => 'audio/vnd.dece.audio',
		'uvd' => 'application/vnd.dece.data',
		'uvf' => 'application/vnd.dece.data',
		'uvg' => 'image/vnd.dece.graphic',
		'uvh' => 'video/vnd.dece.hd',
		'uvi' => 'image/vnd.dece.graphic',
		'uvm' => 'video/vnd.dece.mobile',
		'uvp' => 'video/vnd.dece.pd',
		'uvs' => 'video/vnd.dece.sd',
		'uvt' => 'application/vnd.dece.ttml+xml',
		'uvu' => 'video/vnd.uvvu.mp4',
		'uvv' => 'video/vnd.dece.video',
		'uvva' => 'audio/vnd.dece.audio',
		'uvvd' => 'application/vnd.dece.data',
		'uvvf' => 'application/vnd.dece.data',
		'uvvg' => 'image/vnd.dece.graphic',
		'uvvh' => 'video/vnd.dece.hd',
		'uvvi' => 'image/vnd.dece.graphic',
		'uvvm' => 'video/vnd.dece.mobile',
		'uvvp' => 'video/vnd.dece.pd',
		'uvvs' => 'video/vnd.dece.sd',
		'uvvt' => 'application/vnd.dece.ttml+xml',
		'uvvu' => 'video/vnd.uvvu.mp4',
		'uvvv' => 'video/vnd.dece.video',
		'uvvx' => 'application/vnd.dece.unspecified',
		'uvvz' => 'application/vnd.dece.zip',
		'uvx' => 'application/vnd.dece.unspecified',
		'uvz' => 'application/vnd.dece.zip',
		'vcard' => 'text/vcard',
		'vcd' => 'application/x-cdlink',
		'vcf' => 'text/x-vcard',
		'vcg' => 'application/vnd.groove-vcard',
		'vcs' => 'text/x-vcalendar',
		'vcx' => 'application/vnd.vcx',
		'vis' => 'application/vnd.visionary',
		'viv' => 'video/vnd.vivo',
		'vob' => 'video/x-ms-vob',
		'vor' => 'application/vnd.stardivision.writer',
		'vox' => 'application/x-authorware-bin',
		'vrml' => 'model/vrml',
		'vsd' => 'application/vnd.visio',
		'vsf' => 'application/vnd.vsf',
		'vss' => 'application/vnd.visio',
		'vst' => 'application/vnd.visio',
		'vsw' => 'application/vnd.visio',
		'vtu' => 'model/vnd.vtu',
		'vxml' => 'application/voicexml+xml',
		'w3d' => 'application/x-director',
		'wad' => 'application/x-doom',
		'wav' => 'audio/x-wav',
		'wax' => 'audio/x-ms-wax',
		'wbmp' => 'image/vnd.wap.wbmp',
		'wbs' => 'application/vnd.criticaltools.wbs+xml',
		'wbxml' => 'application/vnd.wap.wbxml',
		'wcm' => 'application/vnd.ms-works',
		'wdb' => 'application/vnd.ms-works',
		'wdp' => 'image/vnd.ms-photo',
		'weba' => 'audio/webm',
		'webm' => 'video/webm',
		'webp' => 'image/webp',
		'wg' => 'application/vnd.pmi.widget',
		'wgt' => 'application/widget',
		'wks' => 'application/vnd.ms-works',
		'wm' => 'video/x-ms-wm',
		'wma' => 'audio/x-ms-wma',
		'wmd' => 'application/x-ms-wmd',
		'wmf' => 'application/x-msmetafile',
		'wml' => 'text/vnd.wap.wml',
		'wmlc' => 'application/vnd.wap.wmlc',
		'wmls' => 'text/vnd.wap.wmlscript',
		'wmlsc' => 'application/vnd.wap.wmlscriptc',
		'wmv' => 'video/x-ms-wmv',
		'wmx' => 'video/x-ms-wmx',
		'wmz' => 'application/x-msmetafile',
		'woff' => 'application/x-font-woff',
		'wpd' => 'application/vnd.wordperfect',
		'wpl' => 'application/vnd.ms-wpl',
		'wps' => 'application/vnd.ms-works',
		'wqd' => 'application/vnd.wqd',
		'wri' => 'application/x-mswrite',
		'wrl' => 'model/vrml',
		'wsdl' => 'application/wsdl+xml',
		'wspolicy' => 'application/wspolicy+xml',
		'wtb' => 'application/vnd.webturbo',
		'wvx' => 'video/x-ms-wvx',
		'x32' => 'application/x-authorware-bin',
		'x3d' => 'model/x3d+xml',
		'x3db' => 'model/x3d+binary',
		'x3dbz' => 'model/x3d+binary',
		'x3dv' => 'model/x3d+vrml',
		'x3dvz' => 'model/x3d+vrml',
		'x3dz' => 'model/x3d+xml',
		'xaml' => 'application/xaml+xml',
		'xap' => 'application/x-silverlight-app',
		'xar' => 'application/vnd.xara',
		'xbap' => 'application/x-ms-xbap',
		'xbd' => 'application/vnd.fujixerox.docuworks.binder',
		'xbm' => 'image/x-xbitmap',
		'xdf' => 'application/xcap-diff+xml',
		'xdm' => 'application/vnd.syncml.dm+xml',
		'xdp' => 'application/vnd.adobe.xdp+xml',
		'xdssc' => 'application/dssc+xml',
		'xdw' => 'application/vnd.fujixerox.docuworks',
		'xenc' => 'application/xenc+xml',
		'xer' => 'application/patch-ops-error+xml',
		'xfdf' => 'application/vnd.adobe.xfdf',
		'xfdl' => 'application/vnd.xfdl',
		'xht' => 'application/xhtml+xml',
		'xhtml' => 'application/xhtml+xml',
		'xhvml' => 'application/xv+xml',
		'xif' => 'image/vnd.xiff',
		'xla' => 'application/vnd.ms-excel',
		'xlam' => 'application/vnd.ms-excel.addin.macroenabled.12',
		'xlc' => 'application/vnd.ms-excel',
		'xlf' => 'application/x-xliff+xml',
		'xlm' => 'application/vnd.ms-excel',
		'xls' => 'application/vnd.ms-excel',
		'xlsb' => 'application/vnd.ms-excel.sheet.binary.macroenabled.12',
		'xlsm' => 'application/vnd.ms-excel.sheet.macroenabled.12',
		'xlsx' => 'application/vnd.openxmlformats-officedocument.spreadsheetml.sheet',
		'xlt' => 'application/vnd.ms-excel',
		'xltm' => 'application/vnd.ms-excel.template.macroenabled.12',
		'xltx' => 'application/vnd.openxmlformats-officedocument.spreadsheetml.template',
		'xlw' => 'application/vnd.ms-excel',
		'xm' => 'audio/xm',
		'xml' => 'application/xml',
		'xo' => 'application/vnd.olpc-sugar',
		'xop' => 'application/xop+xml',
		'xpi' => 'application/x-xpinstall',
		'xpl' => 'application/xproc+xml',
		'xpm' => 'image/x-xpixmap',
		'xpr' => 'application/vnd.is-xpr',
		'xps' => 'application/vnd.ms-xpsdocument',
		'xpw' => 'application/vnd.intercon.formnet',
		'xpx' => 'application/vnd.intercon.formnet',
		'xsl' => 'application/xml',
		'xslt' => 'application/xslt+xml',
		'xsm' => 'application/vnd.syncml+xml',
		'xspf' => 'application/xspf+xml',
		'xul' => 'application/vnd.mozilla.xul+xml',
		'xvm' => 'application/xv+xml',
		'xvml' => 'application/xv+xml',
		'xwd' => 'image/x-xwindowdump',
		'xyz' => 'chemical/x-xyz',
		'xz' => 'application/x-xz',
		'yang' => 'application/yang',
		'yin' => 'application/yin+xml',
		'z1' => 'application/x-zmachine',
		'z2' => 'application/x-zmachine',
		'z3' => 'application/x-zmachine',
		'z4' => 'application/x-zmachine',
		'z5' => 'application/x-zmachine',
		'z6' => 'application/x-zmachine',
		'z7' => 'application/x-zmachine',
		'z8' => 'application/x-zmachine',
		'zaz' => 'application/vnd.zzazz.deck+xml',
		'zip' => 'application/zip',
		'zir' => 'application/vnd.zul',
		'zirz' => 'application/vnd.zul',
		'zmm' => 'application/vnd.handheld-entertainment+xml',
		'123' => 'application/vnd.lotus-1-2-3',
	);

	/**
	 * A map of Internet Media Types to file extensions
	 *
	 * @var array
	 */
	private static $mediaTypeToFileExtension = array(
		'application/andrew-inset' => array('ez'),
		'application/applixware' => array('aw'),
		'application/atom+xml' => array('atom'),
		'application/atomcat+xml' => array('atomcat'),
		'application/atomsvc+xml' => array('atomsvc'),
		'application/ccxml+xml' => array('ccxml'),
		'application/cdmi-capability' => array('cdmia'),
		'application/cdmi-container' => array('cdmic'),
		'application/cdmi-domain' => array('cdmid'),
		'application/cdmi-object' => array('cdmio'),
		'application/cdmi-queue' => array('cdmiq'),
		'application/cu-seeme' => array('cu'),
		'application/davmount+xml' => array('davmount'),
		'application/docbook+xml' => array('dbk'),
		'application/dssc+der' => array('dssc'),
		'application/dssc+xml' => array('xdssc'),
		'application/ecmascript' => array('ecma'),
		'application/emma+xml' => array('emma'),
		'application/epub+zip' => array('epub'),
		'application/exi' => array('exi'),
		'application/font-tdpfr' => array('pfr'),
		'application/gml+xml' => array('gml'),
		'application/gpx+xml' => array('gpx'),
		'application/gxf' => array('gxf'),
		'application/hyperstudio' => array('stk'),
		'application/inkml+xml' => array('ink', 'inkml'),
		'application/ipfix' => array('ipfix'),
		'application/java-archive' => array('jar'),
		'application/java-serialized-object' => array('ser'),
		'application/java-vm' => array('class'),
		'application/javascript' => array('js'),
		'application/json' => array('json'),
		'application/jsonml+json' => array('jsonml'),
		'application/lost+xml' => array('lostxml'),
		'application/mac-binhex40' => array('hqx'),
		'application/mac-compactpro' => array('cpt'),
		'application/mads+xml' => array('mads'),
		'application/marc' => array('mrc'),
		'application/marcxml+xml' => array('mrcx'),
		'application/mathematica' => array('ma', 'nb', 'mb'),
		'application/mathml+xml' => array('mathml'),
		'application/mbox' => array('mbox'),
		'application/mediaservercontrol+xml' => array('mscml'),
		'application/metalink+xml' => array('metalink'),
		'application/metalink4+xml' => array('meta4'),
		'application/mets+xml' => array('mets'),
		'application/mods+xml' => array('mods'),
		'application/mp21' => array('m21', 'mp21'),
		'application/mp4' => array('mp4s'),
		'application/msword' => array('doc', 'dot'),
		'application/mxf' => array('mxf'),
		'application/octet-stream' => array('bin', 'dms', 'lrf', 'mar', 'so', 'dist', 'distz', 'pkg', 'bpk', 'dump', 'elc', 'deploy'),
		'application/oda' => array('oda'),
		'application/oebps-package+xml' => array('opf'),
		'application/ogg' => array('ogx'),
		'application/omdoc+xml' => array('omdoc'),
		'application/onenote' => array('onetoc', 'onetoc2', 'onetmp', 'onepkg'),
		'application/oxps' => array('oxps'),
		'application/patch-ops-error+xml' => array('xer'),
		'application/pdf' => array('pdf'),
		'application/pgp-encrypted' => array('pgp'),
		'application/pgp-signature' => array('asc', 'sig'),
		'application/pics-rules' => array('prf'),
		'application/pkcs10' => array('p10'),
		'application/pkcs7-mime' => array('p7m', 'p7c'),
		'application/pkcs7-signature' => array('p7s'),
		'application/pkcs8' => array('p8'),
		'application/pkix-attr-cert' => array('ac'),
		'application/pkix-cert' => array('cer'),
		'application/pkix-crl' => array('crl'),
		'application/pkix-pkipath' => array('pkipath'),
		'application/pkixcmp' => array('pki'),
		'application/pls+xml' => array('pls'),
		'application/postscript' => array('ai', 'eps', 'ps'),
		'application/prs.cww' => array('cww'),
		'application/pskc+xml' => array('pskcxml'),
		'application/rdf+xml' => array('rdf'),
		'application/reginfo+xml' => array('rif'),
		'application/relax-ng-compact-syntax' => array('rnc'),
		'application/resource-lists+xml' => array('rl'),
		'application/resource-lists-diff+xml' => array('rld'),
		'application/rls-services+xml' => array('rs'),
		'application/rpki-ghostbusters' => array('gbr'),
		'application/rpki-manifest' => array('mft'),
		'application/rpki-roa' => array('roa'),
		'application/rsd+xml' => array('rsd'),
		'application/rss+xml' => array('rss'),
		'application/rtf' => array('rtf'),
		'application/sbml+xml' => array('sbml'),
		'application/scvp-cv-request' => array('scq'),
		'application/scvp-cv-response' => array('scs'),
		'application/scvp-vp-request' => array('spq'),
		'application/scvp-vp-response' => array('spp'),
		'application/sdp' => array('sdp'),
		'application/set-payment-initiation' => array('setpay'),
		'application/set-registration-initiation' => array('setreg'),
		'application/shf+xml' => array('shf'),
		'application/smil+xml' => array('smi', 'smil'),
		'application/sparql-query' => array('rq'),
		'application/sparql-results+xml' => array('srx'),
		'application/srgs' => array('gram'),
		'application/srgs+xml' => array('grxml'),
		'application/sru+xml' => array('sru'),
		'application/ssdl+xml' => array('ssdl'),
		'application/ssml+xml' => array('ssml'),
		'application/tei+xml' => array('tei', 'teicorpus'),
		'application/thraud+xml' => array('tfi'),
		'application/timestamped-data' => array('tsd'),
		'application/vnd.3gpp.pic-bw-large' => array('plb'),
		'application/vnd.3gpp.pic-bw-small' => array('psb'),
		'application/vnd.3gpp.pic-bw-var' => array('pvb'),
		'application/vnd.3gpp2.tcap' => array('tcap'),
		'application/vnd.3m.post-it-notes' => array('pwn'),
		'application/vnd.accpac.simply.aso' => array('aso'),
		'application/vnd.accpac.simply.imp' => array('imp'),
		'application/vnd.acucobol' => array('acu'),
		'application/vnd.acucorp' => array('atc', 'acutc'),
		'application/vnd.adobe.air-application-installer-package+zip' => array('air'),
		'application/vnd.adobe.formscentral.fcdt' => array('fcdt'),
		'application/vnd.adobe.fxp' => array('fxp', 'fxpl'),
		'application/vnd.adobe.xdp+xml' => array('xdp'),
		'application/vnd.adobe.xfdf' => array('xfdf'),
		'application/vnd.ahead.space' => array('ahead'),
		'application/vnd.airzip.filesecure.azf' => array('azf'),
		'application/vnd.airzip.filesecure.azs' => array('azs'),
		'application/vnd.amazon.ebook' => array('azw'),
		'application/vnd.americandynamics.acc' => array('acc'),
		'application/vnd.amiga.ami' => array('ami'),
		'application/vnd.android.package-archive' => array('apk'),
		'application/vnd.anser-web-certificate-issue-initiation' => array('cii'),
		'application/vnd.anser-web-funds-transfer-initiation' => array('fti'),
		'application/vnd.antix.game-component' => array('atx'),
		'application/vnd.apple.installer+xml' => array('mpkg'),
		'application/vnd.apple.mpegurl' => array('m3u8'),
		'application/vnd.aristanetworks.swi' => array('swi'),
		'application/vnd.astraea-software.iota' => array('iota'),
		'application/vnd.audiograph' => array('aep'),
		'application/vnd.blueice.multipass' => array('mpm'),
		'application/vnd.bmi' => array('bmi'),
		'application/vnd.businessobjects' => array('rep'),
		'application/vnd.chemdraw+xml' => array('cdxml'),
		'application/vnd.chipnuts.karaoke-mmd' => array('mmd'),
		'application/vnd.cinderella' => array('cdy'),
		'application/vnd.claymore' => array('cla'),
		'application/vnd.cloanto.rp9' => array('rp9'),
		'application/vnd.clonk.c4group' => array('c4g', 'c4d', 'c4f', 'c4p', 'c4u'),
		'application/vnd.cluetrust.cartomobile-config' => array('c11amc'),
		'application/vnd.cluetrust.cartomobile-config-pkg' => array('c11amz'),
		'application/vnd.commonspace' => array('csp'),
		'application/vnd.contact.cmsg' => array('cdbcmsg'),
		'application/vnd.cosmocaller' => array('cmc'),
		'application/vnd.crick.clicker' => array('clkx'),
		'application/vnd.crick.clicker.keyboard' => array('clkk'),
		'application/vnd.crick.clicker.palette' => array('clkp'),
		'application/vnd.crick.clicker.template' => array('clkt'),
		'application/vnd.crick.clicker.wordbank' => array('clkw'),
		'application/vnd.criticaltools.wbs+xml' => array('wbs'),
		'application/vnd.ctc-posml' => array('pml'),
		'application/vnd.cups-ppd' => array('ppd'),
		'application/vnd.curl.car' => array('car'),
		'application/vnd.curl.pcurl' => array('pcurl'),
		'application/vnd.dart' => array('dart'),
		'application/vnd.data-vision.rdz' => array('rdz'),
		'application/vnd.dece.data' => array('uvf', 'uvvf', 'uvd', 'uvvd'),
		'application/vnd.dece.ttml+xml' => array('uvt', 'uvvt'),
		'application/vnd.dece.unspecified' => array('uvx', 'uvvx'),
		'application/vnd.dece.zip' => array('uvz', 'uvvz'),
		'application/vnd.denovo.fcselayout-link' => array('fe_launch'),
		'application/vnd.dna' => array('dna'),
		'application/vnd.dolby.mlp' => array('mlp'),
		'application/vnd.dpgraph' => array('dpg'),
		'application/vnd.dreamfactory' => array('dfac'),
		'application/vnd.ds-keypoint' => array('kpxx'),
		'application/vnd.dvb.ait' => array('ait'),
		'application/vnd.dvb.service' => array('svc'),
		'application/vnd.dynageo' => array('geo'),
		'application/vnd.ecowin.chart' => array('mag'),
		'application/vnd.enliven' => array('nml'),
		'application/vnd.epson.esf' => array('esf'),
		'application/vnd.epson.msf' => array('msf'),
		'application/vnd.epson.quickanime' => array('qam'),
		'application/vnd.epson.salt' => array('slt'),
		'application/vnd.epson.ssf' => array('ssf'),
		'application/vnd.eszigno3+xml' => array('es3', 'et3'),
		'application/vnd.ezpix-album' => array('ez2'),
		'application/vnd.ezpix-package' => array('ez3'),
		'application/vnd.fdf' => array('fdf'),
		'application/vnd.fdsn.mseed' => array('mseed'),
		'application/vnd.fdsn.seed' => array('seed', 'dataless'),
		'application/vnd.flographit' => array('gph'),
		'application/vnd.fluxtime.clip' => array('ftc'),
		'application/vnd.framemaker' => array('fm', 'frame', 'maker', 'book'),
		'application/vnd.frogans.fnc' => array('fnc'),
		'application/vnd.frogans.ltf' => array('ltf'),
		'application/vnd.fsc.weblaunch' => array('fsc'),
		'application/vnd.fujitsu.oasys' => array('oas'),
		'application/vnd.fujitsu.oasys2' => array('oa2'),
		'application/vnd.fujitsu.oasys3' => array('oa3'),
		'application/vnd.fujitsu.oasysgp' => array('fg5'),
		'application/vnd.fujitsu.oasysprs' => array('bh2'),
		'application/vnd.fujixerox.ddd' => array('ddd'),
		'application/vnd.fujixerox.docuworks' => array('xdw'),
		'application/vnd.fujixerox.docuworks.binder' => array('xbd'),
		'application/vnd.fuzzysheet' => array('fzs'),
		'application/vnd.genomatix.tuxedo' => array('txd'),
		'application/vnd.geogebra.file' => array('ggb'),
		'application/vnd.geogebra.tool' => array('ggt'),
		'application/vnd.geometry-explorer' => array('gex', 'gre'),
		'application/vnd.geonext' => array('gxt'),
		'application/vnd.geoplan' => array('g2w'),
		'application/vnd.geospace' => array('g3w'),
		'application/vnd.gmx' => array('gmx'),
		'application/vnd.google-earth.kml+xml' => array('kml'),
		'application/vnd.google-earth.kmz' => array('kmz'),
		'application/vnd.grafeq' => array('gqf', 'gqs'),
		'application/vnd.groove-account' => array('gac'),
		'application/vnd.groove-help' => array('ghf'),
		'application/vnd.groove-identity-message' => array('gim'),
		'application/vnd.groove-injector' => array('grv'),
		'application/vnd.groove-tool-message' => array('gtm'),
		'application/vnd.groove-tool-template' => array('tpl'),
		'application/vnd.groove-vcard' => array('vcg'),
		'application/vnd.hal+xml' => array('hal'),
		'application/vnd.handheld-entertainment+xml' => array('zmm'),
		'application/vnd.hbci' => array('hbci'),
		'application/vnd.hhe.lesson-player' => array('les'),
		'application/vnd.hp-hpgl' => array('hpgl'),
		'application/vnd.hp-hpid' => array('hpid'),
		'application/vnd.hp-hps' => array('hps'),
		'application/vnd.hp-jlyt' => array('jlt'),
		'application/vnd.hp-pcl' => array('pcl'),
		'application/vnd.hp-pclxl' => array('pclxl'),
		'application/vnd.hydrostatix.sof-data' => array('sfd-hdstx'),
		'application/vnd.ibm.minipay' => array('mpy'),
		'application/vnd.ibm.modcap' => array('afp', 'listafp', 'list3820'),
		'application/vnd.ibm.rights-management' => array('irm'),
		'application/vnd.ibm.secure-container' => array('sc'),
		'application/vnd.iccprofile' => array('icc', 'icm'),
		'application/vnd.igloader' => array('igl'),
		'application/vnd.immervision-ivp' => array('ivp'),
		'application/vnd.immervision-ivu' => array('ivu'),
		'application/vnd.insors.igm' => array('igm'),
		'application/vnd.intercon.formnet' => array('xpw', 'xpx'),
		'application/vnd.intergeo' => array('i2g'),
		'application/vnd.intu.qbo' => array('qbo'),
		'application/vnd.intu.qfx' => array('qfx'),
		'application/vnd.ipunplugged.rcprofile' => array('rcprofile'),
		'application/vnd.irepository.package+xml' => array('irp'),
		'application/vnd.is-xpr' => array('xpr'),
		'application/vnd.isac.fcs' => array('fcs'),
		'application/vnd.jam' => array('jam'),
		'application/vnd.jcp.javame.midlet-rms' => array('rms'),
		'application/vnd.jisp' => array('jisp'),
		'application/vnd.joost.joda-archive' => array('joda'),
		'application/vnd.kahootz' => array('ktz', 'ktr'),
		'application/vnd.kde.karbon' => array('karbon'),
		'application/vnd.kde.kchart' => array('chrt'),
		'application/vnd.kde.kformula' => array('kfo'),
		'application/vnd.kde.kivio' => array('flw'),
		'application/vnd.kde.kontour' => array('kon'),
		'application/vnd.kde.kpresenter' => array('kpr', 'kpt'),
		'application/vnd.kde.kspread' => array('ksp'),
		'application/vnd.kde.kword' => array('kwd', 'kwt'),
		'application/vnd.kenameaapp' => array('htke'),
		'application/vnd.kidspiration' => array('kia'),
		'application/vnd.kinar' => array('kne', 'knp'),
		'application/vnd.koan' => array('skp', 'skd', 'skt', 'skm'),
		'application/vnd.kodak-descriptor' => array('sse'),
		'application/vnd.las.las+xml' => array('lasxml'),
		'application/vnd.llamagraphics.life-balance.desktop' => array('lbd'),
		'application/vnd.llamagraphics.life-balance.exchange+xml' => array('lbe'),
		'application/vnd.lotus-1-2-3' => array('123'),
		'application/vnd.lotus-approach' => array('apr'),
		'application/vnd.lotus-freelance' => array('pre'),
		'application/vnd.lotus-notes' => array('nsf'),
		'application/vnd.lotus-organizer' => array('org'),
		'application/vnd.lotus-screencam' => array('scm'),
		'application/vnd.lotus-wordpro' => array('lwp'),
		'application/vnd.macports.portpkg' => array('portpkg'),
		'application/vnd.mcd' => array('mcd'),
		'application/vnd.medcalcdata' => array('mc1'),
		'application/vnd.mediastation.cdkey' => array('cdkey'),
		'application/vnd.mfer' => array('mwf'),
		'application/vnd.mfmp' => array('mfm'),
		'application/vnd.micrografx.flo' => array('flo'),
		'application/vnd.micrografx.igx' => array('igx'),
		'application/vnd.mif' => array('mif'),
		'application/vnd.mobius.daf' => array('daf'),
		'application/vnd.mobius.dis' => array('dis'),
		'application/vnd.mobius.mbk' => array('mbk'),
		'application/vnd.mobius.mqy' => array('mqy'),
		'application/vnd.mobius.msl' => array('msl'),
		'application/vnd.mobius.plc' => array('plc'),
		'application/vnd.mobius.txf' => array('txf'),
		'application/vnd.mophun.application' => array('mpn'),
		'application/vnd.mophun.certificate' => array('mpc'),
		'application/vnd.mozilla.xul+xml' => array('xul'),
		'application/vnd.ms-artgalry' => array('cil'),
		'application/vnd.ms-cab-compressed' => array('cab'),
		'application/vnd.ms-excel' => array('xls', 'xlm', 'xla', 'xlc', 'xlt', 'xlw'),
		'application/vnd.ms-excel.addin.macroenabled.12' => array('xlam'),
		'application/vnd.ms-excel.sheet.binary.macroenabled.12' => array('xlsb'),
		'application/vnd.ms-excel.sheet.macroenabled.12' => array('xlsm'),
		'application/vnd.ms-excel.template.macroenabled.12' => array('xltm'),
		'application/vnd.ms-fontobject' => array('eot'),
		'application/vnd.ms-htmlhelp' => array('chm'),
		'application/vnd.ms-ims' => array('ims'),
		'application/vnd.ms-lrm' => array('lrm'),
		'application/vnd.ms-officetheme' => array('thmx'),
		'application/vnd.ms-pki.seccat' => array('cat'),
		'application/vnd.ms-pki.stl' => array('stl'),
		'application/vnd.ms-powerpoint' => array('ppt', 'pps', 'pot'),
		'application/vnd.ms-powerpoint.addin.macroenabled.12' => array('ppam'),
		'application/vnd.ms-powerpoint.presentation.macroenabled.12' => array('pptm'),
		'application/vnd.ms-powerpoint.slide.macroenabled.12' => array('sldm'),
		'application/vnd.ms-powerpoint.slideshow.macroenabled.12' => array('ppsm'),
		'application/vnd.ms-powerpoint.template.macroenabled.12' => array('potm'),
		'application/vnd.ms-project' => array('mpp', 'mpt'),
		'application/vnd.ms-word.document.macroenabled.12' => array('docm'),
		'application/vnd.ms-word.template.macroenabled.12' => array('dotm'),
		'application/vnd.ms-works' => array('wps', 'wks', 'wcm', 'wdb'),
		'application/vnd.ms-wpl' => array('wpl'),
		'application/vnd.ms-xpsdocument' => array('xps'),
		'application/vnd.mseq' => array('mseq'),
		'application/vnd.musician' => array('mus'),
		'application/vnd.muvee.style' => array('msty'),
		'application/vnd.mynfc' => array('taglet'),
		'application/vnd.neurolanguage.nlu' => array('nlu'),
		'application/vnd.nitf' => array('ntf', 'nitf'),
		'application/vnd.noblenet-directory' => array('nnd'),
		'application/vnd.noblenet-sealer' => array('nns'),
		'application/vnd.noblenet-web' => array('nnw'),
		'application/vnd.nokia.n-gage.data' => array('ngdat'),
		'application/vnd.nokia.n-gage.symbian.install' => array('n-gage'),
		'application/vnd.nokia.radio-preset' => array('rpst'),
		'application/vnd.nokia.radio-presets' => array('rpss'),
		'application/vnd.novadigm.edm' => array('edm'),
		'application/vnd.novadigm.edx' => array('edx'),
		'application/vnd.novadigm.ext' => array('ext'),
		'application/vnd.oasis.opendocument.chart' => array('odc'),
		'application/vnd.oasis.opendocument.chart-template' => array('otc'),
		'application/vnd.oasis.opendocument.database' => array('odb'),
		'application/vnd.oasis.opendocument.formula' => array('odf'),
		'application/vnd.oasis.opendocument.formula-template' => array('odft'),
		'application/vnd.oasis.opendocument.graphics' => array('odg'),
		'application/vnd.oasis.opendocument.graphics-template' => array('otg'),
		'application/vnd.oasis.opendocument.image' => array('odi'),
		'application/vnd.oasis.opendocument.image-template' => array('oti'),
		'application/vnd.oasis.opendocument.presentation' => array('odp'),
		'application/vnd.oasis.opendocument.presentation-template' => array('otp'),
		'application/vnd.oasis.opendocument.spreadsheet' => array('ods'),
		'application/vnd.oasis.opendocument.spreadsheet-template' => array('ots'),
		'application/vnd.oasis.opendocument.text' => array('odt'),
		'application/vnd.oasis.opendocument.text-master' => array('odm'),
		'application/vnd.oasis.opendocument.text-template' => array('ott'),
		'application/vnd.oasis.opendocument.text-web' => array('oth'),
		'application/vnd.olpc-sugar' => array('xo'),
		'application/vnd.oma.dd2+xml' => array('dd2'),
		'application/vnd.openofficeorg.extension' => array('oxt'),
		'application/vnd.openxmlformats-officedocument.presentationml.presentation' => array('pptx'),
		'application/vnd.openxmlformats-officedocument.presentationml.slide' => array('sldx'),
		'application/vnd.openxmlformats-officedocument.presentationml.slideshow' => array('ppsx'),
		'application/vnd.openxmlformats-officedocument.presentationml.template' => array('potx'),
		'application/vnd.openxmlformats-officedocument.spreadsheetml.sheet' => array('xlsx'),
		'application/vnd.openxmlformats-officedocument.spreadsheetml.template' => array('xltx'),
		'application/vnd.openxmlformats-officedocument.wordprocessingml.document' => array('docx'),
		'application/vnd.openxmlformats-officedocument.wordprocessingml.template' => array('dotx'),
		'application/vnd.osgeo.mapguide.package' => array('mgp'),
		'application/vnd.osgi.dp' => array('dp'),
		'application/vnd.osgi.subsystem' => array('esa'),
		'application/vnd.palm' => array('pdb', 'pqa', 'oprc'),
		'application/vnd.pawaafile' => array('paw'),
		'application/vnd.pg.format' => array('str'),
		'application/vnd.pg.osasli' => array('ei6'),
		'application/vnd.picsel' => array('efif'),
		'application/vnd.pmi.widget' => array('wg'),
		'application/vnd.pocketlearn' => array('plf'),
		'application/vnd.powerbuilder6' => array('pbd'),
		'application/vnd.previewsystems.box' => array('box'),
		'application/vnd.proteus.magazine' => array('mgz'),
		'application/vnd.publishare-delta-tree' => array('qps'),
		'application/vnd.pvi.ptid1' => array('ptid'),
		'application/vnd.quark.quarkxpress' => array('qxd', 'qxt', 'qwd', 'qwt', 'qxl', 'qxb'),
		'application/vnd.realvnc.bed' => array('bed'),
		'application/vnd.recordare.musicxml' => array('mxl'),
		'application/vnd.recordare.musicxml+xml' => array('musicxml'),
		'application/vnd.rig.cryptonote' => array('cryptonote'),
		'application/vnd.rim.cod' => array('cod'),
		'application/vnd.rn-realmedia' => array('rm'),
		'application/vnd.rn-realmedia-vbr' => array('rmvb'),
		'application/vnd.route66.link66+xml' => array('link66'),
		'application/vnd.sailingtracker.track' => array('st'),
		'application/vnd.seemail' => array('see'),
		'application/vnd.sema' => array('sema'),
		'application/vnd.semd' => array('semd'),
		'application/vnd.semf' => array('semf'),
		'application/vnd.shana.informed.formdata' => array('ifm'),
		'application/vnd.shana.informed.formtemplate' => array('itp'),
		'application/vnd.shana.informed.interchange' => array('iif'),
		'application/vnd.shana.informed.package' => array('ipk'),
		'application/vnd.simtech-mindmapper' => array('twd', 'twds'),
		'application/vnd.smaf' => array('mmf'),
		'application/vnd.smart.teacher' => array('teacher'),
		'application/vnd.solent.sdkm+xml' => array('sdkm', 'sdkd'),
		'application/vnd.spotfire.dxp' => array('dxp'),
		'application/vnd.spotfire.sfs' => array('sfs'),
		'application/vnd.stardivision.calc' => array('sdc'),
		'application/vnd.stardivision.draw' => array('sda'),
		'application/vnd.stardivision.impress' => array('sdd'),
		'application/vnd.stardivision.math' => array('smf'),
		'application/vnd.stardivision.writer' => array('sdw', 'vor'),
		'application/vnd.stardivision.writer-global' => array('sgl'),
		'application/vnd.stepmania.package' => array('smzip'),
		'application/vnd.stepmania.stepchart' => array('sm'),
		'application/vnd.sun.xml.calc' => array('sxc'),
		'application/vnd.sun.xml.calc.template' => array('stc'),
		'application/vnd.sun.xml.draw' => array('sxd'),
		'application/vnd.sun.xml.draw.template' => array('std'),
		'application/vnd.sun.xml.impress' => array('sxi'),
		'application/vnd.sun.xml.impress.template' => array('sti'),
		'application/vnd.sun.xml.math' => array('sxm'),
		'application/vnd.sun.xml.writer' => array('sxw'),
		'application/vnd.sun.xml.writer.global' => array('sxg'),
		'application/vnd.sun.xml.writer.template' => array('stw'),
		'application/vnd.sus-calendar' => array('sus', 'susp'),
		'application/vnd.svd' => array('svd'),
		'application/vnd.symbian.install' => array('sis', 'sisx'),
		'application/vnd.syncml+xml' => array('xsm'),
		'application/vnd.syncml.dm+wbxml' => array('bdm'),
		'application/vnd.syncml.dm+xml' => array('xdm'),
		'application/vnd.tao.intent-module-archive' => array('tao'),
		'application/vnd.tcpdump.pcap' => array('pcap', 'cap', 'dmp'),
		'application/vnd.tmobile-livetv' => array('tmo'),
		'application/vnd.trid.tpt' => array('tpt'),
		'application/vnd.triscape.mxs' => array('mxs'),
		'application/vnd.trueapp' => array('tra'),
		'application/vnd.ufdl' => array('ufd', 'ufdl'),
		'application/vnd.uiq.theme' => array('utz'),
		'application/vnd.umajin' => array('umj'),
		'application/vnd.unity' => array('unityweb'),
		'application/vnd.uoml+xml' => array('uoml'),
		'application/vnd.vcx' => array('vcx'),
		'application/vnd.visio' => array('vsd', 'vst', 'vss', 'vsw'),
		'application/vnd.visionary' => array('vis'),
		'application/vnd.vsf' => array('vsf'),
		'application/vnd.wap.wbxml' => array('wbxml'),
		'application/vnd.wap.wmlc' => array('wmlc'),
		'application/vnd.wap.wmlscriptc' => array('wmlsc'),
		'application/vnd.webturbo' => array('wtb'),
		'application/vnd.wolfram.player' => array('nbp'),
		'application/vnd.wordperfect' => array('wpd'),
		'application/vnd.wqd' => array('wqd'),
		'application/vnd.wt.stf' => array('stf'),
		'application/vnd.xara' => array('xar'),
		'application/vnd.xfdl' => array('xfdl'),
		'application/vnd.yamaha.hv-dic' => array('hvd'),
		'application/vnd.yamaha.hv-script' => array('hvs'),
		'application/vnd.yamaha.hv-voice' => array('hvp'),
		'application/vnd.yamaha.openscoreformat' => array('osf'),
		'application/vnd.yamaha.openscoreformat.osfpvg+xml' => array('osfpvg'),
		'application/vnd.yamaha.smaf-audio' => array('saf'),
		'application/vnd.yamaha.smaf-phrase' => array('spf'),
		'application/vnd.yellowriver-custom-menu' => array('cmp'),
		'application/vnd.zul' => array('zir', 'zirz'),
		'application/vnd.zzazz.deck+xml' => array('zaz'),
		'application/voicexml+xml' => array('vxml'),
		'application/widget' => array('wgt'),
		'application/winhlp' => array('hlp'),
		'application/wsdl+xml' => array('wsdl'),
		'application/wspolicy+xml' => array('wspolicy'),
		'application/x-7z-compressed' => array('7z'),
		'application/x-abiword' => array('abw'),
		'application/x-ace-compressed' => array('ace'),
		'application/x-apple-diskimage' => array('dmg'),
		'application/x-authorware-bin' => array('aab', 'x32', 'u32', 'vox'),
		'application/x-authorware-map' => array('aam'),
		'application/x-authorware-seg' => array('aas'),
		'application/x-bcpio' => array('bcpio'),
		'application/x-bittorrent' => array('torrent'),
		'application/x-blorb' => array('blb', 'blorb'),
		'application/x-bzip' => array('bz'),
		'application/x-bzip2' => array('bz2', 'boz'),
		'application/x-cbr' => array('cbr', 'cba', 'cbt', 'cbz', 'cb7'),
		'application/x-cdlink' => array('vcd'),
		'application/x-cfs-compressed' => array('cfs'),
		'application/x-chat' => array('chat'),
		'application/x-chess-pgn' => array('pgn'),
		'application/x-conference' => array('nsc'),
		'application/x-cpio' => array('cpio'),
		'application/x-csh' => array('csh'),
		'application/x-debian-package' => array('deb', 'udeb'),
		'application/x-dgc-compressed' => array('dgc'),
		'application/x-director' => array('dir', 'dcr', 'dxr', 'cst', 'cct', 'cxt', 'w3d', 'fgd', 'swa'),
		'application/x-doom' => array('wad'),
		'application/x-dtbncx+xml' => array('ncx'),
		'application/x-dtbook+xml' => array('dtb'),
		'application/x-dtbresource+xml' => array('res'),
		'application/x-dvi' => array('dvi'),
		'application/x-envoy' => array('evy'),
		'application/x-eva' => array('eva'),
		'application/x-font-bdf' => array('bdf'),
		'application/x-font-ghostscript' => array('gsf'),
		'application/x-font-linux-psf' => array('psf'),
		'application/x-font-otf' => array('otf'),
		'application/x-font-pcf' => array('pcf'),
		'application/x-font-snf' => array('snf'),
		'application/x-font-ttf' => array('ttf', 'ttc'),
		'application/x-font-type1' => array('pfa', 'pfb', 'pfm', 'afm'),
		'application/x-font-woff' => array('woff'),
		'application/x-freearc' => array('arc'),
		'application/x-futuresplash' => array('spl'),
		'application/x-gca-compressed' => array('gca'),
		'application/x-glulx' => array('ulx'),
		'application/x-gnumeric' => array('gnumeric'),
		'application/x-gramps-xml' => array('gramps'),
		'application/x-gtar' => array('gtar'),
		'application/x-hdf' => array('hdf'),
		'application/x-install-instructions' => array('install'),
		'application/x-iso9660-image' => array('iso'),
		'application/x-java-jnlp-file' => array('jnlp'),
		'application/x-latex' => array('latex'),
		'application/x-lzh-compressed' => array('lzh', 'lha'),
		'application/x-mie' => array('mie'),
		'application/x-mobipocket-ebook' => array('prc', 'mobi'),
		'application/x-ms-application' => array('application'),
		'application/x-ms-shortcut' => array('lnk'),
		'application/x-ms-wmd' => array('wmd'),
		'application/x-ms-wmz' => array('wmz'),
		'application/x-ms-xbap' => array('xbap'),
		'application/x-msaccess' => array('mdb'),
		'application/x-msbinder' => array('obd'),
		'application/x-mscardfile' => array('crd'),
		'application/x-msclip' => array('clp'),
		'application/x-msdownload' => array('exe', 'dll', 'com', 'bat', 'msi'),
		'application/x-msmediaview' => array('mvb', 'm13', 'm14'),
		'application/x-msmetafile' => array('wmf', 'wmz', 'emf', 'emz'),
		'application/x-msmoney' => array('mny'),
		'application/x-mspublisher' => array('pub'),
		'application/x-msschedule' => array('scd'),
		'application/x-msterminal' => array('trm'),
		'application/x-mswrite' => array('wri'),
		'application/x-netcdf' => array('nc', 'cdf'),
		'application/x-nzb' => array('nzb'),
		'application/x-pkcs12' => array('p12', 'pfx'),
		'application/x-pkcs7-certificates' => array('p7b', 'spc'),
		'application/x-pkcs7-certreqresp' => array('p7r'),
		'application/x-rar-compressed' => array('rar'),
		'application/x-research-info-systems' => array('ris'),
		'application/x-sh' => array('sh'),
		'application/x-shar' => array('shar'),
		'application/x-shockwave-flash' => array('swf'),
		'application/x-silverlight-app' => array('xap'),
		'application/x-sql' => array('sql'),
		'application/x-stuffit' => array('sit'),
		'application/x-stuffitx' => array('sitx'),
		'application/x-subrip' => array('srt'),
		'application/x-sv4cpio' => array('sv4cpio'),
		'application/x-sv4crc' => array('sv4crc'),
		'application/x-t3vm-image' => array('t3'),
		'application/x-tads' => array('gam'),
		'application/x-tar' => array('tar'),
		'application/x-tcl' => array('tcl'),
		'application/x-tex' => array('tex'),
		'application/x-tex-tfm' => array('tfm'),
		'application/x-texinfo' => array('texinfo', 'texi'),
		'application/x-tgif' => array('obj'),
		'application/x-ustar' => array('ustar'),
		'application/x-wais-source' => array('src'),
		'application/x-x509-ca-cert' => array('der', 'crt'),
		'application/x-xfig' => array('fig'),
		'application/x-xliff+xml' => array('xlf'),
		'application/x-xpinstall' => array('xpi'),
		'application/x-xz' => array('xz'),
		'application/x-zmachine' => array('z1', 'z2', 'z3', 'z4', 'z5', 'z6', 'z7', 'z8'),
		'application/xaml+xml' => array('xaml'),
		'application/xcap-diff+xml' => array('xdf'),
		'application/xenc+xml' => array('xenc'),
		'application/xhtml+xml' => array('xhtml', 'xht'),
		'application/xml' => array('xml', 'xsl'),
		'application/xml-dtd' => array('dtd'),
		'application/xop+xml' => array('xop'),
		'application/xproc+xml' => array('xpl'),
		'application/xslt+xml' => array('xslt'),
		'application/xspf+xml' => array('xspf'),
		'application/xv+xml' => array('mxml', 'xhvml', 'xvml', 'xvm'),
		'application/yang' => array('yang'),
		'application/yin+xml' => array('yin'),
		'application/zip' => array('zip'),
		'audio/adpcm' => array('adp'),
		'audio/basic' => array('au', 'snd'),
		'audio/midi' => array('mid', 'midi', 'kar', 'rmi'),
		'audio/mp4' => array('mp4a'),
		'audio/mpeg' => array('mpga', 'mp2', 'mp2a', 'mp3', 'm2a', 'm3a'),
		'audio/ogg' => array('oga', 'ogg', 'spx'),
		'audio/s3m' => array('s3m'),
		'audio/silk' => array('sil'),
		'audio/vnd.dece.audio' => array('uva', 'uvva'),
		'audio/vnd.digital-winds' => array('eol'),
		'audio/vnd.dra' => array('dra'),
		'audio/vnd.dts' => array('dts'),
		'audio/vnd.dts.hd' => array('dtshd'),
		'audio/vnd.lucent.voice' => array('lvp'),
		'audio/vnd.ms-playready.media.pya' => array('pya'),
		'audio/vnd.nuera.ecelp4800' => array('ecelp4800'),
		'audio/vnd.nuera.ecelp7470' => array('ecelp7470'),
		'audio/vnd.nuera.ecelp9600' => array('ecelp9600'),
		'audio/vnd.rip' => array('rip'),
		'audio/webm' => array('weba'),
		'audio/x-aac' => array('aac'),
		'audio/x-aiff' => array('aif', 'aiff', 'aifc'),
		'audio/x-caf' => array('caf'),
		'audio/x-flac' => array('flac'),
		'audio/x-matroska' => array('mka'),
		'audio/x-mpegurl' => array('m3u'),
		'audio/x-ms-wax' => array('wax'),
		'audio/x-ms-wma' => array('wma'),
		'audio/x-pn-realaudio' => array('ram', 'ra'),
		'audio/x-pn-realaudio-plugin' => array('rmp'),
		'audio/x-wav' => array('wav'),
		'audio/xm' => array('xm'),
		'chemical/x-cdx' => array('cdx'),
		'chemical/x-cif' => array('cif'),
		'chemical/x-cmdf' => array('cmdf'),
		'chemical/x-cml' => array('cml'),
		'chemical/x-csml' => array('csml'),
		'chemical/x-xyz' => array('xyz'),
		'image/bmp' => array('bmp'),
		'image/cgm' => array('cgm'),
		'image/g3fax' => array('g3'),
		'image/gif' => array('gif'),
		'image/ief' => array('ief'),
		'image/jpeg' => array('jpeg', 'jpg', 'jpe'),
		'image/ktx' => array('ktx'),
		'image/png' => array('png'),
		'image/prs.btif' => array('btif'),
		'image/sgi' => array('sgi'),
		'image/svg+xml' => array('svg', 'svgz'),
		'image/tiff' => array('tiff', 'tif'),
		'image/vnd.adobe.photoshop' => array('psd'),
		'image/vnd.dece.graphic' => array('uvi', 'uvvi', 'uvg', 'uvvg'),
		'image/vnd.djvu' => array('djvu', 'djv'),
		'image/vnd.dvb.subtitle' => array('sub'),
		'image/vnd.dwg' => array('dwg'),
		'image/vnd.dxf' => array('dxf'),
		'image/vnd.fastbidsheet' => array('fbs'),
		'image/vnd.fpx' => array('fpx'),
		'image/vnd.fst' => array('fst'),
		'image/vnd.fujixerox.edmics-mmr' => array('mmr'),
		'image/vnd.fujixerox.edmics-rlc' => array('rlc'),
		'image/vnd.ms-modi' => array('mdi'),
		'image/vnd.ms-photo' => array('wdp'),
		'image/vnd.net-fpx' => array('npx'),
		'image/vnd.wap.wbmp' => array('wbmp'),
		'image/vnd.xiff' => array('xif'),
		'image/webp' => array('webp'),
		'image/x-3ds' => array('3ds'),
		'image/x-cmu-raster' => array('ras'),
		'image/x-cmx' => array('cmx'),
		'image/x-freehand' => array('fh', 'fhc', 'fh4', 'fh5', 'fh7'),
		'image/x-icon' => array('ico'),
		'image/x-mrsid-image' => array('sid'),
		'image/x-pcx' => array('pcx'),
		'image/x-pict' => array('pic', 'pct'),
		'image/x-portable-anymap' => array('pnm'),
		'image/x-portable-bitmap' => array('pbm'),
		'image/x-portable-graymap' => array('pgm'),
		'image/x-portable-pixmap' => array('ppm'),
		'image/x-rgb' => array('rgb'),
		'image/x-tga' => array('tga'),
		'image/x-xbitmap' => array('xbm'),
		'image/x-xpixmap' => array('xpm'),
		'image/x-xwindowdump' => array('xwd'),
		'message/rfc822' => array('eml', 'mime'),
		'model/iges' => array('igs', 'iges'),
		'model/mesh' => array('msh', 'mesh', 'silo'),
		'model/vnd.collada+xml' => array('dae'),
		'model/vnd.dwf' => array('dwf'),
		'model/vnd.gdl' => array('gdl'),
		'model/vnd.gtw' => array('gtw'),
		'model/vnd.mts' => array('mts'),
		'model/vnd.vtu' => array('vtu'),
		'model/vrml' => array('wrl', 'vrml'),
		'model/x3d+binary' => array('x3db', 'x3dbz'),
		'model/x3d+vrml' => array('x3dv', 'x3dvz'),
		'model/x3d+xml' => array('x3d', 'x3dz'),
		'text/cache-manifest' => array('appcache'),
		'text/calendar' => array('ics', 'ifb'),
		'text/css' => array('css'),
		'text/csv' => array('csv'),
		'text/html' => array('html', 'htm'),
		'text/n3' => array('n3'),
		'text/plain' => array('txt', 'text', 'conf', 'def', 'list', 'log', 'in'),
		'text/prs.lines.tag' => array('dsc'),
		'text/richtext' => array('rtx'),
		'text/sgml' => array('sgml', 'sgm'),
		'text/tab-separated-values' => array('tsv'),
		'text/troff' => array('t', 'tr', 'roff', 'man', 'me', 'ms'),
		'text/turtle' => array('ttl'),
		'text/uri-list' => array('uri', 'uris', 'urls'),
		'text/vcard' => array('vcard'),
		'text/vnd.curl' => array('curl'),
		'text/vnd.curl.dcurl' => array('dcurl'),
		'text/vnd.curl.mcurl' => array('mcurl'),
		'text/vnd.curl.scurl' => array('scurl'),
		'text/vnd.dvb.subtitle' => array('sub'),
		'text/vnd.fly' => array('fly'),
		'text/vnd.fmi.flexstor' => array('flx'),
		'text/vnd.graphviz' => array('gv'),
		'text/vnd.in3d.3dml' => array('3dml'),
		'text/vnd.in3d.spot' => array('spot'),
		'text/vnd.sun.j2me.app-descriptor' => array('jad'),
		'text/vnd.wap.wml' => array('wml'),
		'text/vnd.wap.wmlscript' => array('wmls'),
		'text/x-asm' => array('s', 'asm'),
		'text/x-c' => array('c', 'cc', 'cxx', 'cpp', 'h', 'hh', 'dic'),
		'text/x-fortran' => array('f', 'for', 'f77', 'f90'),
		'text/x-java-source' => array('java'),
		'text/x-nfo' => array('nfo'),
		'text/x-opml' => array('opml'),
		'text/x-pascal' => array('p', 'pas'),
		'text/x-setext' => array('etx'),
		'text/x-sfv' => array('sfv'),
		'text/x-uuencode' => array('uu'),
		'text/x-vcalendar' => array('vcs'),
		'text/x-vcard' => array('vcf'),
		'video/3gpp' => array('3gp'),
		'video/3gpp2' => array('3g2'),
		'video/h261' => array('h261'),
		'video/h263' => array('h263'),
		'video/h264' => array('h264'),
		'video/jpeg' => array('jpgv'),
		'video/jpm' => array('jpm', 'jpgm'),
		'video/mj2' => array('mj2', 'mjp2'),
		'video/mp4' => array('mp4', 'mp4v', 'mpg4'),
		'video/mpeg' => array('mpeg', 'mpg', 'mpe', 'm1v', 'm2v'),
		'video/ogg' => array('ogv'),
		'video/quicktime' => array('qt', 'mov'),
		'video/vnd.dece.hd' => array('uvh', 'uvvh'),
		'video/vnd.dece.mobile' => array('uvm', 'uvvm'),
		'video/vnd.dece.pd' => array('uvp', 'uvvp'),
		'video/vnd.dece.sd' => array('uvs', 'uvvs'),
		'video/vnd.dece.video' => array('uvv', 'uvvv'),
		'video/vnd.dvb.file' => array('dvb'),
		'video/vnd.fvt' => array('fvt'),
		'video/vnd.mpegurl' => array('mxu', 'm4u'),
		'video/vnd.ms-playready.media.pyv' => array('pyv'),
		'video/vnd.uvvu.mp4' => array('uvu', 'uvvu'),
		'video/vnd.vivo' => array('viv'),
		'video/webm' => array('webm'),
		'video/x-f4v' => array('f4v'),
		'video/x-fli' => array('fli'),
		'video/x-flv' => array('flv'),
		'video/x-m4v' => array('m4v'),
		'video/x-matroska' => array('mkv', 'mk3d', 'mks'),
		'video/x-mng' => array('mng'),
		'video/x-ms-asf' => array('asf', 'asx'),
		'video/x-ms-vob' => array('vob'),
		'video/x-ms-wm' => array('wm'),
		'video/x-ms-wmv' => array('wmv'),
		'video/x-ms-wmx' => array('wmx'),
		'video/x-ms-wvx' => array('wvx'),
		'video/x-msvideo' => array('avi'),
		'video/x-sgi-movie' => array('movie'),
		'video/x-smv' => array('smv'),
		'x-conference/x-cooltalk' => array('ice'),
	);

	/**
	 * Returns a Media Type based on the filename extension
	 *
	 * @param  string $filename Filename to determine the media type for, for example "index.html"
	 * @return string The IANA Internet Media Type
	 * @api
	 */
	static public function getMediaTypeFromFilename($filename) {
		$pathinfo = pathinfo(strtolower($filename));
		if (!isset($pathinfo['extension'])) {
			return 'application/octet-stream';
		} else {
			return isset(self::$extensionToMediaType[$pathinfo['extension']]) ? self::$extensionToMediaType[$pathinfo['extension']] : 'application/octet-stream';
		}
	}

	/**
	 * Returns the primary filename extension based on the given Media Type.
	 *
	 * @param string $mediaType The IANA Internet Media Type, for example "text/html"
	 * @return string The corresponding filename extension, for example "html"
	 * @api
	 */
	static public function getFilenameExtensionFromMediaType($mediaType) {
		return isset(self::$mediaTypeToFileExtension[$mediaType]) ? self::$mediaTypeToFileExtension[$mediaType][0] : '';
	}

	/**
	 * Returns all possible filename extensions based on the given Media Type.
	 *
	 * @param string $mediaType The IANA Internet Media Type, for example "text/html"
	 * @return array The corresponding filename extensions, for example ("html", "htm")
	 * @api
	 */
	static public function getFilenameExtensionsFromMediaType($mediaType) {
		return isset(self::$mediaTypeToFileExtension[$mediaType]) ? self::$mediaTypeToFileExtension[$mediaType] : array();
	}

	/**
	 * Parses a RFC 2616 Media Type and returns its parts in an associative array.
	 *
	 * media-type = type "/" subtype *( ";" parameter)
	 *
	 * The media type "text/html; charset=UTF-8" would be parsed and returned with
	 * the following array keys and values:
	 *
	 * "type" => the type as a string, "text"
	 * "subtype" => the subtype as a string, "html"
	 * "parameters" => an array of parameter names and values, array("charset" => "UTF-8")
	 *
	 * @param string $rawMediaType The raw media type, for example "application/json; charset=UTF-8"
	 * @return array An associative array with parsed information
	 */
	static public function parseMediaType($rawMediaType) {
		preg_match(self::PATTERN_SPLITMEDIATYPE, $rawMediaType, $matches);
		$result = array();
		$result['type'] = isset($matches['type']) ? $matches['type'] : '';
		$result['subtype'] = isset($matches['subtype']) ? $matches['subtype'] : '';
		$result['parameters'] = array();

		if (isset($matches['parameters'])) {
			foreach (Arrays::trimExplode(';', $matches['parameters']) as $parameter) {
				$pieces = explode('=', $parameter);
				if (count($pieces) === 2) {
					$name = trim($pieces[0]);
					$result['parameters'][$name] = trim($pieces[1]);
				}
			}
		}
		return $result;
	}

	/**
	 * Checks if the given media range and the media type match.
	 *
	 * The pattern used by this function splits each into media type and subtype
	 * and ignores possible additional parameters or extensions. The prepared types
	 * and subtypes are then compared and wildcards are considered in this process.
	 *
	 * Media ranges are explained in RFC 2616, section 14.1. "Accept".
	 *
	 * @param string $mediaRange The media range, for example "text/*"
	 * @param string $mediaType The media type to match against, for example "text/html"
	 * @return boolean TRUE if both match, FALSE if they don't match or either of them is invalid
	 */
	static public function mediaRangeMatches($mediaRange, $mediaType) {
		preg_match(self::PATTERN_SPLITMEDIARANGE, $mediaRange, $mediaRangeMatches);
		preg_match(self::PATTERN_SPLITMEDIATYPE, $mediaType, $mediaTypeMatches);
		if ($mediaRangeMatches === array() || $mediaTypeMatches === array()) {
			return FALSE;
		}

		$typeMatches = ($mediaRangeMatches['type'] === '*' || $mediaRangeMatches['type'] === $mediaTypeMatches['type']);
		$subtypeMatches = ($mediaRangeMatches['subtype'] === '*' || $mediaRangeMatches['subtype'] === $mediaTypeMatches['subtype']);

		return ($typeMatches && $subtypeMatches);
	}

	/**
	 * Strips off any parameters from the given media type and returns just the type
	 * and subtype in the format "type/subtype".
	 *
	 * @param string $rawMediaType The full media type, for example "application/json; charset=UTF-8"
	 * @return string Just the type and subtype, for example "application/json"
	 */
	static public function trimMediaType($rawMediaType) {
		$pieces = self::parseMediaType($rawMediaType);
		return trim(sprintf('%s/%s', $pieces['type'], $pieces['subtype']), '/') ?: NULL;
	}
}
